/* this is a work in progress. :) */

/***********************************************
 * Method descriptions
 ***********************************************/

main .method-detail {
  display: grid;
  grid-template-columns: 1fr auto;
  justify-content: space-between;
}

main .method-header,
main .method-controls,
.attribute-method-heading {
  padding: 0.5em;
  /* border: 1px solid var(--highlight-color); */
  background: var(--table-header-background-color);
  line-height: 1.6;
}

.attribute-method-heading .attribute-access-type {
  float: right;
}

main .method-header {
  border-right: none;
  border-radius: 4px 0 0 4px;
}

main .method-heading :any-link {
  text-decoration: none;
}

main .method-controls {
  border-left: none;
  border-radius: 0 4px 4px 0;
}

main .method-description, main .aliases {
  grid-column: 1 / span 2;
  padding-left: 1em;
}

@media (max-width: 700px) {
  main .method-header, main .method-controls, main .method-description {
    grid-column: 1 / span 2;
    margin: 0;
  }
  main .method-controls {
    background: none;
  }
}

/***********************************************
 * Description lists
 ***********************************************/

main dt {
  margin-bottom: 0; /* override rdoc 6.8 */
  float: unset;     /* override rdoc 6.8 */
  line-height: 1.5; /* matches `main p` */
}

main dl.note-list dt {
  margin-right: 1em;
  float: left;
}

main dl.note-list dt:has(+ dt) {
  margin-right: 0.25em;
}

main dl.note-list dt:has(+ dt)::after {
  content: ', ';
  font-weight: normal;
}

main dd {
  margin: 0 0 1em 1em;
}

main dd p:first-child {
  margin-top: 0;
}
