require_relative '../../spec_helper'
require 'date'

describe "DateTime.iso8601" do
  it "needs to be reviewed for spec completeness"
end

describe "DateTime#iso8601" do
  it "needs to be reviewed for spec completeness"
end
